"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fc = require("fast-check");
var O = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
var R = require("fp-ts/lib/Record");
var t = require("io-ts");
var objectTypes = ['ExactType', 'InterfaceType', 'PartialType'];
function getArbitrary(codec, customArbitrary) {
    return pipeable_1.pipe(O.fromNullable(customArbitrary), O.chain(function (ca) { return ca(codec); }), O.getOrElse(function () {
        var type = codec;
        switch (type._tag) {
            case 'StringType':
                return fc.string();
            case 'UndefinedType':
                return fc.constant(undefined);
            case 'NumberType':
                return fc.float();
            case 'BooleanType':
                return fc.boolean();
            case 'LiteralType':
                return fc.constant(type.value);
            case 'InterfaceType':
                return fc.record(R.record.map(type.props, function (prop) { return getArbitrary(prop, customArbitrary); }));
            case 'PartialType':
                return fc.record(R.record.map(type.props, function (type) {
                    return getArbitrary(t.union([t.undefined, type]), customArbitrary);
                }));
            case 'UnionType':
                return fc.oneof.apply(fc, type.types.map(function (t) { return getArbitrary(t, customArbitrary); }));
            case 'KeyofType':
                return fc.oneof.apply(fc, Object.keys(type.keys).map(fc.constant));
            case 'NullType':
                return fc.constant(null);
            case 'TupleType':
                return fc.tuple.apply(fc, type.types.map(function (t) { return getArbitrary(t, customArbitrary); }));
            case 'UnknownType':
                return fc.anything();
            case 'VoidType':
                return fc.constant(undefined);
            case 'ArrayType':
                return fc.array(getArbitrary(type.type, customArbitrary));
            case 'ExactType':
                return getArbitrary(type.type, customArbitrary);
            case 'RefinementType':
                return getArbitrary(type.type, customArbitrary).filter(type.predicate);
            case 'IntersectionType':
                var isObjectIntersection = objectTypes.includes(type.types[0]._tag);
                return isObjectIntersection
                    ? fc.tuple.apply(fc, type.types.map(function (t) { return getArbitrary(t, customArbitrary); })).map(function (values) { return Object.assign.apply(Object, [{}].concat(values)); })
                        .filter(type.is)
                    : fc.oneof.apply(fc, type.types.map(function (t) { return getArbitrary(t, customArbitrary); })).filter(type.is);
        }
        throw new Error("Codec not supported: " + codec);
    }));
}
exports.getArbitrary = getArbitrary;
